# -*- coding: utf-8 -*-
import os
from django.conf import settings
from django.utils.importlib import import_module


class BaseTask(object):
    """
    Base interface for ci tasks
    """
    option_list = []

    def __init__(self, test_labels, options):
        self.test_labels = test_labels

    def setup_test_environment(self, **kwargs):
        pass

    def teardown_test_environment(self, **kwargs):
        pass

    def before_suite_run(self, **kwargs):
        pass

    def after_suite_run(self, **kwargs):
        pass

    def build_suite(self, suite, **kwargs):
        pass


def get_apps_under_test(test_labels, all_apps=False):
    """
    Convert test_lables for apps names

    all_apps - if test_labels empty, ignore white list,
    and returns all installed apps
    """
    if not test_labels:
        if hasattr(settings, 'PROJECT_APPS') and not all_apps:
            apps = settings.PROJECT_APPS
        else:
            apps = settings.INSTALLED_APPS
    else:
        apps = [app for app in settings.INSTALLED_APPS \
                    for label in test_labels \
                    if app == label.split('.')[0] or app.endswith('.%s' % label.split('.')[0])]
    return apps


def get_apps_locations(test_labels, all_apps=False):
    """
    Returns list of paths to tested apps
    """
    return [os.path.dirname(os.path.normpath(import_module(app_name).__file__)) \
            for app_name in get_apps_under_test(test_labels, all_apps)]


def get_app_location(app_module):
    """
    Returns path to app
    """
    return os.path.dirname(os.path.normpath(app_module.__file__))




# -*- coding: utf-8 -*-
from django.conf import settings
from django_jenkins.management.commands import TaskListCommand


class Command(TaskListCommand):
    help = "Run CI process"
    args = '[appname ...]'

    def get_task_list(self):
        return getattr(settings, 'JENKINS_TASKS',
                       ('django_jenkins.tasks.run_pylint',
                        'django_jenkins.tasks.with_coverage',
                        'django_jenkins.tasks.django_tests',))



# -*- coding: utf-8; mode: django -*-
import os
import traceback
from datetime import datetime
from itertools import groupby
from xml.sax.saxutils import XMLGenerator
from xml.sax.xmlreader import AttributesImpl
from django.conf import settings
from django.test.simple import DjangoTestSuiteRunner, reorder_suite
from django.test.testcases import TestCase
from django.utils.unittest import TestSuite, TextTestResult, TextTestRunner
from django_jenkins import signals
from django_jenkins.functions import total_seconds

try:
    from django.utils.encoding import smart_text
except ImportError:
    from django.utils.encoding import smart_unicode as smart_text


STDOUT_LINE = '\nStdout:\n%s'
STDERR_LINE = '\nStderr:\n%s'


class TestInfo(object):
    class RESULT(object):
        SUCCESS = 0
        ERROR = 1
        FAILURE = 2
        EXPECTED_FAILURE = 3
        UNEXPECTED_SUCCESS = 4
        SKIPPED = 5

    __slots__ = ('test_method', 'start_time', 'end_time',
                 'err', 'stdout', 'stderr', 'result', 'reason')

    def __init__(self, **kwargs):
        for slot_name in self.__slots__:
            setattr(self, slot_name, None)
        for key, value in kwargs.items():
            setattr(self, key, value)


class XMLTestResult(TextTestResult):
    """
    Dumps xml junit output as well as text
    """
    def __init__(self, *args, **kwargs):
        super(XMLTestResult, self).__init__(*args, **kwargs)
        self.testInfos = []
        self.currentTestInfo = None

    def startTestRun(self):
        """
        Called once before any tests are executed.
        """
        super(XMLTestResult, self).startTestRun()

    def startTest(self, test):
        """
        Called when the given test is about to be run
        """
        self.currentTestInfo = TestInfo(test_method=test, start_time=datetime.now())
        super(XMLTestResult, self).startTest(test)

    def stopTest(self, test):
        """
        Called when the given test has been run
        """
        self.currentTestInfo.end_time = datetime.now()
        self.currentTestInfo.stdout = self._stdout_buffer.getvalue()
        self.currentTestInfo.stderr = self._stderr_buffer.getvalue()
        self.testInfos.append(self.currentTestInfo)
        super(XMLTestResult, self).stopTest(test)

    def addError(self, test, err):
        """
        Called when an error has occurred. 'err' is a tuple of values as
        returned by sys.exc_info()
        """
        self.currentTestInfo.result = TestInfo.RESULT.ERROR
        self.currentTestInfo.err = err
        super(XMLTestResult, self).addError(test, err)
        signals.test_error.send(sender=self, test=test, err=err)

    def addFailure(self, test, err):
        """
        Called when an error has occurred. 'err' is a tuple of values as
        returned by sys.exc_info().
        """
        self.currentTestInfo.result = TestInfo.RESULT.FAILURE
        self.currentTestInfo.err = err
        super(XMLTestResult, self).addFailure(test, err)
        signals.test_failure.send(sender=self, test=test, err=err)

    def addSuccess(self, test):
        """
        Called when a test has completed successfully
        """
        self.currentTestInfo.result = TestInfo.RESULT.SUCCESS
        super(XMLTestResult, self).addSuccess(test)
        signals.test_success.send(sender=self, test=test)

    def addSkip(self, test, reason):
        """
        Called when a test is skipped.
        """
        self.currentTestInfo.result = TestInfo.RESULT.SKIPPED
        self.currentTestInfo.reason = reason
        super(XMLTestResult, self).addSkip(test, reason)

    def addExpectedFailure(self, test, err):
        """
        Called when an expected failure/error occured.
        """
        self.currentTestInfo.result = TestInfo.RESULT.EXPECTED_FAILURE
        self.currentTestInfo.err = err
        super(XMLTestResult, self).addExpectedFailure(test, err)
        signals.test_expected_failure.send(sender=self, test=test, err=err)

    def addUnexpectedSuccess(self, test):
        """
        Called when a test was expected to fail, but succeed.
        """
        self.currentTestInfo.result = TestInfo.RESULT.UNEXPECTED_SUCCESS
        super(XMLTestResult, self).addUnexpectedSuccess(test)
        signals.test_unexpected_success.send(sender=self, test=test)

    def _exc_info_to_string(self, err, test):
        """
        Converts a sys.exc_info()-style tuple of values into a string.
        """
        exctype, value, tb = err
        # Skip test runner traceback levels
        while tb and self._is_relevant_tb_level(tb):
            tb = tb.tb_next
        if exctype is test.failureException:
            # Skip assert*() traceback levels
            length = self._count_relevant_tb_levels(tb)
            msgLines = traceback.format_exception(exctype, value, tb, length)
        else:
            msgLines = traceback.format_exception(exctype, value, tb)

        if self.buffer:
            output = self._stdout_buffer.getvalue()
            error = self._stderr_buffer.getvalue()
            if output:
                if not output.endswith('\n'):
                    output += '\n'
                msgLines.append(STDOUT_LINE % output)
            if error:
                if not error.endswith('\n'):
                    error += '\n'
                msgLines.append(STDERR_LINE % error)
        return ''.join(msgLines)

    def test_method_name(self, test):
        """
        Returns the test method name.
        """
        test_id = test.id()
        return test_id.split('.')[-1]

    def test_case_name(self, test):
        """
        Returns test case name
        """
        testcase = type(test)
        module = testcase.__module__ + '.'
        if module == '__main__.':
            module = ''
        return module + testcase.__name__

    def dump_xml(self, output_dir):
        """
        Dumps test result to xml
        """
        self.buffer = False

        if not os.path.exists(output_dir):
            os.makedirs(output_dir)

        with open(os.path.join(output_dir, 'junit.xml'), 'w') as output:
            document = XMLGenerator(output, 'utf-8')
            document.startDocument()
            document.startElement('testsuites', AttributesImpl({}))

            suites = groupby(self.testInfos, key=lambda test_info: self.test_case_name(test_info.test_method))
            for suite_name, suite in suites:
                document.startElement('testsuite', AttributesImpl({'name' : suite_name}))

                for test_info in suite:
                    document.startElement('testcase', AttributesImpl({
                        'classname' : suite_name,
                        'name' : self.test_method_name(test_info.test_method),
                        'time' : '%3f' % total_seconds(test_info.end_time - test_info.start_time)
                    }))

                    if test_info.result == TestInfo.RESULT.ERROR:
                        document.startElement('error', AttributesImpl({
                            'message' : smart_text(test_info.err[1])
                        }))
                        document.characters(self._exc_info_to_string(test_info.err, test_info.test_method))
                        document.endElement('error')
                    elif test_info.result == TestInfo.RESULT.FAILURE:
                        document.startElement('failure', AttributesImpl({
                            'message' : smart_text(test_info.err[1])
                        }))
                        document.characters(self._exc_info_to_string(test_info.err, test_info.test_method))
                        document.endElement('failure')
                    elif test_info.result == TestInfo.RESULT.UNEXPECTED_SUCCESS:
                        document.startElement('error', AttributesImpl({
                            'message' : 'Unexpected success'
                        }))
                        document.endElement('error')
                    elif test_info.result == TestInfo.RESULT.SKIPPED:
                        document.startElement('skipped', AttributesImpl({}))
                        document.characters(test_info.reason)
                        document.endElement('skipped')

                    if test_info.stdout:
                        document.startElement('system-out', AttributesImpl({}))
                        document.characters(test_info.stdout)
                        document.endElement('system-out')

                    if test_info.stderr:
                        document.startElement('system-err', AttributesImpl({}))
                        document.characters(test_info.stderr)
                        document.endElement('system-err')
                    document.endElement('testcase')

                document.endElement('testsuite')

            document.endElement('testsuites')
            document.endDocument()


class CITestSuiteRunner(DjangoTestSuiteRunner):
    """
    Continuous integration test runner
    """
    def __init__(self, output_dir, with_reports=True, **kwargs):
        super(CITestSuiteRunner, self).__init__(**kwargs)
        self.with_reports = with_reports
        self.output_dir = output_dir

    def setup_test_environment(self, **kwargs):
        super(CITestSuiteRunner, self).setup_test_environment()
        signals.setup_test_environment.send(sender=self)

    def teardown_test_environment(self, **kwargs):
        super(CITestSuiteRunner, self).teardown_test_environment()
        signals.teardown_test_environment.send(sender=self)

    def setup_databases(self):
        if 'south' in settings.INSTALLED_APPS:
            from south.management.commands import patch_for_test_db_setup  # pylint: disable=F0401
            patch_for_test_db_setup()
        return super(CITestSuiteRunner, self).setup_databases()

    def build_suite(self, test_labels, extra_tests=None, **kwargs):
        suite = TestSuite()
        signals.build_suite.send(sender=self, suite=suite)
        return reorder_suite(suite, (TestCase,))

    def run_suite(self, suite, **kwargs):
        signals.before_suite_run.send(sender=self)
        result = TextTestRunner(buffer=True,
                resultclass=XMLTestResult,
                verbosity=self.verbosity
                ).run(suite)
        if self.with_reports:
            result.dump_xml(self.output_dir)
        signals.after_suite_run.send(sender=self)
        return result



# -*- coding: utf-8 -*-
import os
from optparse import make_option
from django.conf import settings
from django_jenkins.tasks import BaseTask
from django.test import LiveServerTestCase
from lettuce.django import harvest_lettuces
from lettuce import Runner
from lettuce import registry


class Task(BaseTask):
    option_list = [
        make_option("--lettuce-server",
               dest="lettuce-server",
               help="do not start runserver for lettuce tests",
               default=False),
    ]

    def __init__(self, test_labels, options):
        super(Task, self).__init__(test_labels, options)
        if not self.test_labels:
            if hasattr(settings, 'PROJECT_APPS') and not options['test_all']:
                self.test_labels = [app_name.split('.')[-1] for app_name in settings.PROJECT_APPS]

        self.lettuce_server = options['lettuce-server']
        self.output_dir = options['output_dir']

    def setup_test_environment(self, **kwargs):
        if self.lettuce_server:
            from lettuce.django import server
            self.server = server
            self.server.start()

    def teardown_test_environment(self, **kwargs):
        if self.lettuce_server:
            self.server.stop()

    def build_suite(self, suite, **kwargs):
        paths = harvest_lettuces(self.test_labels)

        if not os.path.exists(self.output_dir):
            os.makedirs(self.output_dir)

        for app_path, app_module in paths:
            runner = Runner(app_path,
                            enable_xunit=True,
                            xunit_filename=os.path.join(self.output_dir, 'lettuce.xml'))

            suite.addTest(LettuceTestCase(runner, app_module))

        return suite


class LettuceTestCase(LiveServerTestCase):
    def __init__(self, runner, app_module, *args, **kwargs):
        super(LettuceTestCase, self).__init__(*args, **kwargs)
        self.runner = runner
        self.app_module = app_module

    def runTest(self):
        registry.call_hook('before_each', 'app', self.app_module)
        result = self.runner.run()
        registry.call_hook('after_each', 'app', self.app_module, result)



# -*- coding: utf-8 -*-
from django.conf.urls.defaults import *

urlpatterns = patterns('',
     url(r'^test_click/$', 'django.views.generic.simple.direct_to_template',
         {'template': 'test_app/wm_test_click.html'}, name='wm_test_click')
)



# -*- coding: utf-8 -*-
import os.path
import subprocess


class CalledProcessError(subprocess.CalledProcessError):
    def __init__(self, returncode, cmd, output=None):
        super(CalledProcessError, self).__init__(returncode, cmd)
        self.output = output

    def __str__(self):
        return "Command '%s' returned non-zero exit status %d\nOutput:\n%s" % (self.cmd, self.returncode, self.output)


def relpath(path, start=os.path.curdir):
    """
    Return a relative version of a path

    Backport from Python2.7
    """
    if not path:
        raise ValueError("no path specified")

    start_list = os.path.abspath(start).split(os.path.sep)
    path_list = os.path.abspath(path).split(os.path.sep)

    # Work out how much of the filepath is shared by start and path.
    i = len(os.path.commonprefix([start_list, path_list]))

    rel_list = [os.path.pardir] * (len(start_list) - i) + path_list[i:]
    if not rel_list:
        return os.path.curdir
    return os.path.join(*rel_list)


def check_output(*popenargs, **kwargs):
    """
    Backport from Python2.7
    """
    if 'stdout' in kwargs or 'stderr' in kwargs:
        raise ValueError('stdout or stderr argument not allowed, it will be overridden.')
    process = subprocess.Popen(stdout=subprocess.PIPE, stderr=subprocess.PIPE, *popenargs, **kwargs)
    output, err = process.communicate()
    retcode = process.poll()
    if retcode:
        cmd = kwargs.get("args")
        if cmd is None:
            cmd = popenargs[0]
        raise CalledProcessError(retcode, cmd, output=output + '\n' + err)
    return output


def find_first_existing_executable(exe_list):
    """
    Accepts list of [('executable_file_path', 'options')],
    Returns first working executable_file_path
    """
    for filepath, opts in exe_list:
        try:
            proc = subprocess.Popen([filepath, opts], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            proc.communicate()
        except OSError:
            pass
        else:
            return filepath


def total_seconds(delta):
    """
    Backport timedelta.total_seconds() from Python 2.7
    """
    return delta.days * 86400.0 + delta.seconds + delta.microseconds * 1e-6




# -*- coding: utf-8 -*-
from optparse import make_option
from django_jenkins.management.commands import TaskListCommand

class Command(TaskListCommand):
    help = "Run lettuce tests"

    def get_task_list(self):
        return ('django_jenkins.tasks.lettuce_tests',)



# -*- coding: utf-8 -*-
# pylint: disable=W0201
import os
from optparse import make_option
from django.conf import settings
from django.core.management import call_command, CommandError
from django_jenkins.tasks import BaseTask, get_apps_under_test


class Task(BaseTask):
    # Using options from modelviz.py / graph_models.py, prefixed with graphmodels[_-]
    option_list = [
        make_option('--graphmodels-disable-fields', action='store_true', dest='graphmodels_disable_fields',
                    help='Do not show the class member fields'),
        make_option('--graphmodels-group-models', action='store_true', dest='graphmodels_group_models',
                    help='Group models together respective to their application'),
        make_option('--graphmodels-all-applications', action='store_true', dest='graphmodels_all_applications',
                    help='Automatically include all applications from INSTALLED_APPS'),
        make_option('--graphmodels-output', action='store', dest='graphmodels_outputfile',
                    help='Render output file. Type of output dependend on file extensions. Use png or jpg to render graph to image.'),
        make_option('--graphmodels-layout', action='store', dest='graphmodels_layout', default='dot',
                    help='Layout to be used by GraphViz for visualization. Layouts: circo dot fdp neato nop nop1 nop2 twopi'),
        make_option('--graphmodels-verbose-names', action='store_true', dest='graphmodels_verbose_names',
                    help='Use verbose_name of models and fields'),
        make_option('--graphmodels-language', action='store', dest='graphmodels_language',
                    help='Specify language used for verbose_name localization'),
        make_option('--graphmodels-exclude-columns', action='store', dest='graphmodels_exclude_columns',
                    help='Exclude specific column(s) from the graph. Can also load exclude list from file.'),
        make_option('--graphmodels-exclude-models', action='store', dest='graphmodels_exclude_models',
                    help='Exclude specific model(s) from the graph. Can also load exclude list from file.'),
        make_option('--graphmodels-inheritance', action='store_true', dest='graphmodels_inheritance',
                    help='Include inheritance arrows'),
    ]

    def checkdeps(self):
        if 'django_extensions' not in settings.INSTALLED_APPS:
            if self.options.get('fail_without_error', False):
                return False
            else:
                raise CommandError("django-extensions is required to execute this command")

        try:
            import pygraphviz  # noqa
        except ImportError:
            if self.options.get('fail_without_error', False):
                return False
            else:
                raise CommandError("pygraphviz is required to execute this command")

        return True

    def __init__(self, test_labels, options):
        super(Task, self).__init__(test_labels, options)
        self.options = options

        # Rename options, for the call to the existing graph_models command
        for key, value in self.options.items():
            if key.startswith('graphmodels_'):
                newkey = key[12:]
                del self.options[key]
                self.options[newkey] = value

        # Merge in possibles settings file options, if the option is unset (None)
        fromfile = getattr(settings, 'GRAPH_MODELS', {})
        fromfile.update(dict((k, v) for k, v in self.options.iteritems() if v))
        self.options.update(fromfile)

        if self.options['test_all']:
            self.options['all_applications'] = True

        # Place the file in the correct place
        output_dir = self.options['output_dir']
        if not os.path.exists(output_dir):
            os.makedirs(output_dir)
        self.options['outputfile'] = os.path.join(output_dir, 'models.png')

        if not self.checkdeps():
            return

    def teardown_test_environment(self, **kwargs):
        # Get the list of PROJECT_APPS if nothing specified any we don't want everything
        if len(self.test_labels) < 1 and not self.options['all_applications']:
            under = get_apps_under_test(self.test_labels, self.options['all_applications'])
            self.test_labels = [label.split('.')[-1] for label in under]

        call_command('graph_models', *self.test_labels, **self.options)



# -*- coding: utf-8 -*-
from optparse import make_option
from django_jenkins.management.commands import TaskListCommand


class Command(TaskListCommand):
    help = "Run pyflakes over project apps"
    args = '[appname ...]'
    option_list = TaskListCommand.option_list + (
        make_option('--pyflakes-file-output', action='store_true', dest='pyflakes_file_output', default=False,
            help='Store pep8 report in file'),
    )

    def get_task_list(self):
        return ('django_jenkins.tasks.run_pyflakes',)



# -*- coding: utf-8 -*-
# pylint: disable=W0201
import os
from optparse import make_option
from coverage.control import coverage
from django.conf import settings
from django.utils.importlib import import_module
from django_jenkins.tasks import BaseTask, get_apps_under_test


class Task(BaseTask):
    option_list = [make_option("--coverage-rcfile",
                               dest="coverage_rcfile",
                               default="",
                               help="Specify configuration file."),
                   make_option("--coverage-html-report",
                               dest="coverage_html_report_dir",
                               default="",
                               help="Directory to which HTML coverage report should be written. If not specified, no report is generated."),
                   make_option("--coverage-no-branch-measure",
                               action="store_false", default=True,
                               dest="coverage_measure_branch",
                               help="Don't measure branch coverage."),
                   make_option("--coverage-with-migrations",
                               action="store_true", default=False,
                               dest="coverage_with_migrations",
                               help="Don't measure migrations coverage."),
                   make_option("--coverage-exclude", action="append",
                               default=[], dest="coverage_excludes",
                               help="Module name to exclude")]

    def __init__(self, test_labels, options):
        super(Task, self).__init__(test_labels, options)
        self.test_apps = get_apps_under_test(test_labels, options['test_all'])
        self.output_dir = options['output_dir']
        self.with_migrations = options.get('coverage_with_migrations',
                                           getattr(settings, 'COVERAGE_WITH_MIGRATIONS', False))
        self.html_dir = options.get('coverage_html_report_dir',
                                    getattr(settings, 'COVERAGE_REPORT_HTML_OUTPUT_DIR', ''))
        self.branch = options.get('coverage_measure_branch',
                                  getattr(settings, 'COVERAGE_MEASURE_BRANCH', True))

        self.exclude_locations = []
        for modname in options.get('coverage_excludes',
                                   getattr(settings, 'COVERAGE_EXCLUDES', [])):
            try:
                self.exclude_locations.append(os.path.dirname(import_module(modname).__file__))
            except ImportError:
                pass

        self.coverage = coverage(branch=self.branch,
                                 source=self.test_apps,
                                 config_file=options.get('coverage_rcfile') or Task.default_config_path())

    def setup_test_environment(self, **kwargs):
        self.coverage.start()

    def teardown_test_environment(self, **kwargs):
        self.coverage.stop()

        morfs = [filename for filename in self.coverage.data.measured_files()
                 if self.want_file(filename)]

        if not os.path.exists(self.output_dir):
            os.makedirs(self.output_dir)
        self.coverage.xml_report(morfs=morfs, outfile=os.path.join(self.output_dir, 'coverage.xml'))

        if self.html_dir:
            self.coverage.html_report(morfs=morfs, directory=self.html_dir)

    def want_file(self, filename):
        if not self.with_migrations and '/migrations/' in filename:
            return False
        for location in self.exclude_locations:
            if filename.startswith(location):
                return False

        return True

    @staticmethod
    def default_config_path():
        rcfile = getattr(settings, 'COVERAGE_RCFILE', 'coverage.rc')
        if os.path.exists(rcfile):
            return rcfile
        return None



# -*- coding: utf-8 -*-
"""
Discover tests from all test*.py files in app subdirectories
"""
import os
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.db.models import get_app, get_apps
from django.test.simple import build_test
from django.utils.unittest import defaultTestLoader
from django_jenkins.tasks import BaseTask, get_app_location


def build_suite(app):
    discovery_root = get_app_location(app)
    if os.path.isdir(os.path.join(discovery_root, 'tests')):
         discovery_root = os.path.join(discovery_root, 'tests')
         top_level_dir = discovery_root
    else:
        top_level_dir = discovery_root
        for _ in range(0, app.__name__.count('.')):
            top_level_dir = os.path.dirname(top_level_dir)
    return defaultTestLoader.discover(discovery_root, top_level_dir=top_level_dir)


class Task(BaseTask):
    def __init__(self, test_labels, options):
        super(Task, self).__init__(test_labels, options)
        if not self.test_labels:
            if hasattr(settings, 'PROJECT_APPS') and not options['test_all']:
                self.test_labels = [app_name.split('.')[-1] for app_name in settings.PROJECT_APPS]

    def build_suite(self, suite, **kwargs):
        if self.test_labels:
            for label in self.test_labels:
                if '.' in label:
                    suite.addTest(build_test(label))
                else:
                    try:
                        app = get_app(label)
                        suite.addTest(build_suite(app))
                    except ImproperlyConfigured:
                        pass
        else:
            for app in get_apps():
                suite.addTest(build_suite(app))




# -*- coding: utf-8 -*-
import os
import sys
from optparse import make_option
from django_jenkins.functions import check_output
from django_jenkins.tasks import BaseTask, get_apps_locations


class Task(BaseTask):
    option_list = [
        make_option("--sloccount-with-migrations",
                    action="store_true", default=False,
                    dest="sloccount_with_migrations",
                    help="Count migrations sloc.")]
    
    def __init__(self, test_labels, options):
        super(Task, self).__init__(test_labels, options)
        self.test_all = options['test_all']
        self.with_migrations = options['sloccount_with_migrations']

        if options.get('sloccount_file_output', True):
            output_dir = options['output_dir']
            if not os.path.exists(output_dir):
                os.makedirs(output_dir)
            self.output = open(os.path.join(output_dir, 'sloccount.report'), 'w')
        else:
            self.output = sys.stdout 
        
                                  
    def teardown_test_environment(self, **kwargs):
        locations = get_apps_locations(self.test_labels, self.test_all)  

        report_output = check_output(
            ['sloccount', "--duplicates", "--wide", "--details"] + locations)
        report_output = report_output.decode('utf-8')

        if self.with_migrations:
            self.output.write(report_output.decode('utf-8'))
        else:
            for line in report_output.splitlines():
                if '/migrations/' in line:
                    continue
                self.output.write(line)
                self.output.write('\n')




# -*- coding: utf-8 -*-
# pylint: disable=W0201, W0141
"""
Build suite with normal django tests
"""
from django.test.simple import build_suite, build_test
from django.db.models import get_app, get_apps
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django_jenkins.tasks import BaseTask


class Task(BaseTask):
    def __init__(self, test_labels, options):
        super(Task, self).__init__(test_labels, options)
        if not self.test_labels:
            if hasattr(settings, 'PROJECT_APPS') and not options['test_all']:
                self.test_labels = [app_name.split('.')[-1] for app_name in settings.PROJECT_APPS]

    def build_suite(self, suite, **kwargs):
        if self.test_labels:
            for label in self.test_labels:
                if '.' in label:
                    suite.addTest(build_test(label))
                else:
                    try:
                        app = get_app(label)
                        suite.addTest(build_suite(app))
                    except ImproperlyConfigured:
                        pass
        else:
            for app in get_apps():
                suite.addTest(build_suite(app))



# -*- coding: utf-8 -*-
import sys
from os import path
from django.core.management import execute_manager

PROJECT_ROOT = path.dirname(path.abspath(path.dirname(__file__)))
sys.path.insert(0, PROJECT_ROOT)

try:
    import tests.settings
except ImportError:
    import sys
    sys.stderr.write("Error: Can't find the file 'settings.py' in the directory")
    sys.exit(1)

if __name__ == "__main__":
    if len(sys.argv) == 1:
        sys.argv += ['test']
    execute_manager(tests.settings)



# -*- coding: utf-8 -*-
import os
import inspect
import sys
from optparse import make_option, OptionGroup
from django.conf import settings
from django.core.management.base import BaseCommand
from django.utils.importlib import import_module
from django_jenkins import signals
from django_jenkins.runner import CITestSuiteRunner

def get_runner():
    if hasattr(settings, 'JENKINS_TEST_RUNNER'):
        test_path = settings.JENKINS_TEST_RUNNER.split('.')
        # Allow for Python 2.5 relative paths
        if len(test_path) > 1:
            test_module_name = '.'.join(test_path[:-1])
        else:
            test_module_name = '.'
        test_module = __import__(test_module_name, {}, {}, test_path[-1])
        test_runner = getattr(test_module, test_path[-1])

        if not issubclass(test_runner, CITestSuiteRunner):
            raise ValueError('Your custom TestRunner should extend the CITestSuiteRunner class.')
        return test_runner
    else:
        return CITestSuiteRunner


class TaskListCommand(BaseCommand):
    """
    Run list of predifined tasks from command line
    """
    requires_model_validation = False  # if True, breaks coverage of models.py files

    option_list = BaseCommand.option_list + (
        make_option('--all', action='store_true', dest='test_all', default=False,
            help='Ignore PROJECT_APPS settings and run through all INSTALLED_APPS'),
        make_option('--interactive', action='store_true', dest='interactive', default=False,
            help='Allow to ask user input'),
        make_option('--debug', action='store_true', dest='debug', default=False,
            help='Do not intercept stdout and stderr, friendly for console debuggers'),
        make_option('--output-dir', dest='output_dir', default="reports",
            help='Report files directory'),
        make_option('--liveserver',
            action='store', dest='liveserver', default=None,
            help='Overrides the default address where the live server (used '
                 'with LiveServerTestCase) is expected to run from. The '
                 'default value is localhost:8081.')
    )

    def __init__(self):
        super(TaskListCommand, self).__init__()
        self.tasks_cls = [import_module(module_name).Task for module_name in self.get_task_list()]

    def handle(self, *test_labels, **options):
        # options
        if options.get('liveserver') is not None:
            os.environ['DJANGO_LIVE_TEST_SERVER_ADDRESS'] = options['liveserver']
            del options['liveserver']

        # instantiate tasks
        self.tasks = self.get_tasks(*test_labels, **options)

        # subscribe
        for signal_name, signal in inspect.getmembers(signals, predicate=lambda obj: obj):
            for task in self.tasks:
                signal_handler = getattr(task, signal_name, None)
                if signal_handler:
                    signal.connect(signal_handler)

        # run
        test_runner_cls = get_runner()
        test_runner = test_runner_cls(
            output_dir=options['output_dir'],
            interactive=options['interactive'],
            debug=options['debug'],
            verbosity=int(options.get('verbosity', 1)),
            with_reports=options.get('with_reports', True))

        if test_runner.run_tests(test_labels):
            sys.exit(1)

    def get_tasks(self, *test_labels, **options):
        """
        Instantiate all task instances
        """
        return [task_cls(test_labels, options) for task_cls in self.tasks_cls]

    def get_task_list(self):
        """
        Return list of task modules for command

        Subclasses should override this method
        """
        return []

    def create_parser(self, *args):
        """
        Extend the option list with tasks specific options
        """
        parser = super(TaskListCommand, self).create_parser(*args)

        for task_cls in self.tasks_cls:
            option_group = OptionGroup(parser, task_cls.__module__, "")

            if task_cls.option_list:
                for option in task_cls.option_list:
                    option_group.add_option(option)
                parser.add_option_group(option_group)

        return parser



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import codecs
from os import path
from setuptools import setup


read = lambda filepath: codecs.open(filepath, 'r', 'utf-8').read()


setup(
    name = 'django-jenkins',
    version = '0.14.0',
    author = 'Mikhail Podgurskiy',
    author_email = 'kmmbvnr@gmail.com',
    description = 'Plug and play continuous integration with django and jenkins',
    long_description=read(path.abspath(path.join(path.dirname(__file__), 'README.rst'))),
    license = 'LGPL',
    platforms = ['Any'],
    keywords = ['pyunit', 'unittest', 'testrunner', 'hudson', 'jenkins', 
                'django', 'pylint', 'pep8', 'pyflakes', 'csslint', 'jshint',
                'coverage'],
    url = 'http://github.com/kmmbvnr/django-jenkins',
    classifiers = [
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Software Development :: Testing'
    ],
    install_requires=[
        'Django>=1.3',
        'coverage>=3.4',
        'pylint>=0.23',
    ],
    packages = ['django_jenkins', 'django_jenkins.management', 'django_jenkins.tasks', 'django_jenkins.management.commands'],
    package_data={'django_jenkins': ['tasks/pylint.rc', 'tasks/jshint_runner.js', 'tasks/jshint/jshint.js', 'tasks/csslint/release/csslint-rhino.js', 'tasks/csslint/release/npm/cli.js','tasks/csslint/release/npm/lib/csslint-node.js']},
    zip_safe = False,
    include_package_data = True
)
